void CRotaryCtrl::OnDraw(
		CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
	// ** Set up the background and foreground brushes
	CBrush brForeGnd(RGB(0,255,0));
	CBrush brBackGnd(TranslateColor(AmbientBackColor()));

	// ** Draw the control background 
	pdc->FillRect(rcBounds, &brBackGnd);

	CBrush* pOldBrush = pdc->SelectObject(&brForeGnd);

	// ** Calculate the relative positions and mid point
	CPoint ptRelative = m_ptClicked - rcBounds.TopLeft();
	CPoint ptMid(rcBounds.Width()/2,rcBounds.Height()/2);

	// ** Find offset from the middle
	double dRelX = ptRelative.x - ptMid.x;
	double dRelY = ptRelative.y - ptMid.y;

	// ** Use trig to find the angle by T=O/A
	double dAngle = atan2(dRelY,dRelX);
	double dRadX = (double)ptMid.x * 0.9;
	double dRadY = (double)ptMid.y * 0.9;

	// ** Find a point on the radius of the knob
	int dXPos = ptMid.x + (int)(cos(dAngle) * dRadX);
	int dYPos = ptMid.y + (int)(sin(dAngle) * dRadY);

	// ** Set the notch point position
	CPoint ptKnob = CPoint(dXPos,dYPos) + rcBounds.TopLeft();

	// ** Set a rect and draw the notch circle
	CRect rcPoint(ptKnob-CSize(4,4),CSize(8,8));
	pdc->Ellipse(rcPoint);

	// ** Draw the main rotary circle
	pdc->Ellipse(ptMid.x-(int)dRadX,ptMid.y-(int)dRadY,
						ptMid.x+(int)dRadX,ptMid.y+(int)dRadY);

	// ** Draw a line from the center to the knotch
	pdc->MoveTo(ptMid);
	pdc->LineTo(ptKnob);

	pdc->SelectObject(pOldBrush);
}
